#ifndef lint
static char SCCSid[] = "@(#) ./comm/schedule/distschd.c 12/01/93";
#endif

/*
   This file contains routines to sort a communication schedule so that 
   sends and receives are ordered by distance.  This is a simple method to
   ensure that contention is reduced in the case where messages are sent
   to distant nodes.  Also, because of the way in which ties are broken, 
   nodes will tend not to send at each other.
 */
#include "tools.h" 
#include "comm/schedule/sch.h"

SCentry *SCCreate( n )
int n;
{
SCentry *new;
new = (SCentry *) MALLOC( n * sizeof(SCentry) );    CHKPTRN(new);

return new;
}

void SCDestroy( sc )
SCentry *sc;
{
FREE( sc );
}

void SCSet( sc, i, is_send, partner, partner_rank, len, priv )
SCentry *sc;
int     i, is_send, partner, partner_rank, len;
void    *priv;
{
sc[i].is_send      = is_send;
sc[i].partner      = partner;
sc[i].partner_rank = partner_rank;
sc[i].len          = len;
sc[i].order        = -1;
sc[i].private      = priv;
}
	
int SCdistsort( a, b )
SCentry *a, *b;
{
int da, db;

da = PIdistance( PImytid, a->partner );
db = PIdistance( PImytid, b->partner );
if (da != db) return da - db;
return a->order - b->order;
}

/*
   SCByDistance - Sort communications operations by distance

   Note:
   The algorithm is

   Sort by (distance,destination_processor + current processor) (stablely)

   Input Parameters:
.  sc      - array of entries
.  n       - number of entries
 */
void SCByDistance( sc, n )
SCentry *sc;
int     n;
{
int i;	
qsort( (char *)sc, n, sizeof(SCentry), (int (*)())SCdistsort );
for (i=0; i<n; i++) sc[i].order = i;
}

/*
   This routine sorts a communication schedule so that 
   sends and receives are matched up.  That is, if all participating 
   processors contain valid schedules, and they use this routine to sort
   the order of those schedules, then they are guarenteed not to deadlock
   even in the absence of any message buffering.

   This is done by using a simple bit mask operation.  It may NOT lead to
   an optimal schedule but it DOES yield an optimal schedule for the case of
   nearest neighbor communication on a grid with an even number of processors
   in each direction.
 */   
void SCCollectSRPair( sc, n, ps )
SCentry *sc;
int     n;
ProcSet *ps;
{
int i, mask, nbr, issend, cnt, myrank;

/* Order them so that send/receives are paired */
cnt    = 0;
myrank = PSMYPROCID(ps);
mask   = 0x1;
while (mask && cnt < n) {
    for (i=0; i<n; i++) {
    	if (sc[i].order == -1) {
    	    nbr    = sc[i].partner_rank;
    	    issend = sc[i].is_send;

	    /* I'm odd, nbr is even, and doing a receive */
    	    if ( !(myrank & mask) && (nbr & mask) && !issend)
    	        sc[i].order = cnt++;
    	    /* I'm even, nbr is odd, and doing a send */
    	    else if ( (myrank & mask) && !(nbr & mask) && issend)
    	    	sc[i].order = cnt++;
    	    }
        }
    for (i=0; i<n; i++) {
    	if (sc[i].order == -1) {
    	    nbr    = sc[i].partner_rank;
    	    issend = sc[i].is_send; 

	    /* I'm odd, nbr is even, and doing a send */
    	    if ( !(myrank & mask) && (nbr & mask) && issend)
    	        sc[i].order = cnt++;
    	    /* I'm even, nbr is odd, and doing a receive */
    	    else if ( (myrank & mask) && !(nbr & mask) && !issend)
    	    	sc[i].order = cnt++;
    	    }
        }
    mask <<= 1;
    }
/* Order value is set; still need to SORT by order */
/* ... */    
}



